@@ -1,5 +1,6 @@
 {-# OPTIONS_HADDOCK prune #-}
 {-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE DerivingStrategies #-}
 {-# LANGUAGE LambdaCase #-}
@@ -10,6 +11,11 @@
 {-# LANGUAGE UnboxedTuples #-}
 {-# LANGUAGE ViewPatterns #-}
 
+#include "MachDeps.h"
+#if WORD_SIZE_IN_BITS != 64
+#error "ppad-secp256k1 requires a 64-bit architecture"
+#endif
+
 -- |
 -- Module: Crypto.Curve.Secp256k1
 -- Copyright: (c) 2024 Jared Tobin
@@ -137,8 +143,7 @@ pattern Z :: Limb4
 pattern Z = (# Limb 0##, Limb 0##, Limb 0##, Limb 0## #)
 
 pattern P :: Limb4 -> Limb4 -> Limb4 -> Projective
-pattern P x y z =
-  Projective (C.Montgomery x) (C.Montgomery y) (C.Montgomery z)
+pattern P x y z = Projective (C.Montgomery x) (C.Montgomery y) (C.Montgomery z)
 {-# COMPLETE P #-}
 
 -- utilities ------------------------------------------------------------------
@@ -285,10 +290,7 @@ unroll32 (Wider (# w0, w1, w2, w3 #)) =
 
 -- modQ via conditional subtraction
 modQ :: Wider -> Wider
-modQ x =
-  let !(Wider xw) = x
-      !(Wider qw) = _CURVE_Q
-  in  W.select x (x - _CURVE_Q) (CT.not (W.lt# xw qw))
+modQ x = W.select x (x - _CURVE_Q) (CT.not (W.lt x _CURVE_Q))
 {-# INLINABLE modQ #-}
 
 -- bytewise xor
@@ -442,8 +444,8 @@ even_y_vartime p = case affine p of
 
 -- Constant-time selection of Projective points.
 select_proj :: Projective -> Projective -> CT.Choice -> Projective
-select_proj (P ax ay az) (P bx by bz) c =
-  P (W.select# ax bx c) (W.select# ay by c) (W.select# az bz c)
+select_proj (Projective ax ay az) (Projective bx by bz) c =
+  Projective (C.select ax bx c) (C.select ay by c) (C.select az bz c)
 {-# INLINE select_proj #-}
 
 -- unboxed internals ----------------------------------------------------------
@@ -607,7 +609,7 @@ mul_wnaf# ctxArray ctxW ls
 
               !abs_b       = CT.select_word# b0 (Exts.minusWord# mnum b0) bor
               !is_zero     = CT.from_word_eq# b0 0##
-              !c0          = CT.from_word# (Exts.and# w 1##)
+              !c0          = CT.from_bit# (Exts.and# w 1##)
               !off_nz      = Exts.minusWord# (Exts.plusWord# off0 abs_b) 1##
               !off         = CT.select_word# off0 off_nz (CT.not is_zero)
 
